//******************************************************
// TP2 (CAPAX-MD) : Identification MCC
// V1.0 Le 14/08/2018
// CAPAX SYSTEMS by www.infoconsult.tn
//******************************************************

# define MCC_pin 9 // Commande de la vitesse du "MCC" 
# define HALL_pin 2 // Lecture des implusion du capteur à effet HALL_pin.
# define LED_V 3 // Led Verte
# define LED_O 5 // Led Orange
# define LED_R 6 // Led Rouge

unsigned long time; // Déclaration de la variable "time".
unsigned long AncienTemps = 0; // Déclaration de la variable "AncienTemps".
unsigned long Duree = 0; // Déclaration de la variable "Duree" de long.
unsigned long TempsCalcul; // Déclaration de la variable "TempsCalcul" de type long pour déterminer le temps de calcul.
unsigned long AncienTempsCalcul; // Déclaration de la variable "AncienTempsCalcul" de type long pour mémoriser l'ancien de calcul précédent.

// le décalage ne se fait que sur 1023 étapes
// la 1024 valeur doit être nulle
// la séquence la plus longue démarre avec 1000111000 = 0x238 = 568
// Pour avoir cette séquence au milieu, on démarre avec 1100100101 = 0x325 = 805
// Pour avoir cette séquence à la fin, on démarre avec 0111111111 = 0x1FF = 511

boolean rad[10] = {0, 1, 1, 1, 1, 1, 1, 1, 1, 1}; // Initialisation du rad : registre à décalage
boolean sbpa_b[1024];

byte j; // Déclaration de la variable 'j'de type byte (entier)
int k; // Déclaration de la variable 'k'de type integer (entier)
int i; // Déclaration de la variable 'i'de type integer (entier) de valeur égale à 0,cette variable correspond au compteur
int VitNominale = 255; // Echelon 5V
int Te = 15000; // Déclaration de la variable "Te" période d'échantillonnage [microseconde]
int taille  = 1024; // Déclaration de la variable "taille" de la SBPA (1024 = 2^10)
unsigned long Vit; // Déclaration de la variable "Vit" : vitesse de rotation du moteur
float rds_mille = 62831853071.8; // conversion de la vitesse en rd/s, le vitesse étant mesurée en tr/microseconde.
// Cette variable est multipliée par mille pour des raisons de transfert vers Matlab
int cmdPWM;

void setup() {
  pinMode(MCC_pin, OUTPUT);
  pinMode(HALL_pin, INPUT);
  pinMode(LED_V, OUTPUT);
  pinMode(LED_O, OUTPUT);
  pinMode(LED_R, OUTPUT);

  attachInterrupt(digitalPinToInterrupt(HALL_pin), TimeCount, FALLING);
  Serial.begin(250000);
  analogWrite(MCC_pin, 0);
  digitalWrite(LED_O, HIGH); // Démarrage de l'excitation par le signal SBPA, LED Orange allumée
  delay(5000); // Temporisation pour le démarrage afin de s'assurer que le CD-ROM est à l'arrêt.
  digitalWrite(LED_O, LOW); // Démarrage de l'excitation par le signal SBPA
  digitalWrite(LED_R, HIGH); // LED Rouge allumée
  sbpa_b[0] = rad[0];
  cmdPWM = VitNominale * sbpa_b[0]; //Calculer la commande pour le moteur
  analogWrite(MCC_pin, cmdPWM); // Envoi de la commande au moteur
  // analogWrite(MCC_pin, VitNominale) Si désire que l'excitation soit un échelon 

  for (i = 1; i < taille; i++) {
    AncienTempsCalcul = micros();
    sbpa_b[i] = 1; // // principe du OU exclusif , si si a =b, s = 0 , si a=/b, s = 1
    if (rad[6] == rad[9]) { // Dans le cas d'une SBPA d'ordre 10, les registres 7 et 10 sont bouclés entre eux 
      sbpa_b[i] = 0 ;
    }
    cmdPWM = VitNominale * sbpa_b[i];
    analogWrite(MCC_pin, cmdPWM);
    if (Duree == 0) { // Si la Duree=0 alors la vitesse =0, pour éviter la division par zéro de la vitesse
      Vit = 0;
    }
    else {
      Vit = rds_mille / Duree; // Conversion de la vitesse mesurée en rd/s
    }
    for (k = 0; k <= 24; k += 8) {
      Serial.write(Vit >> k); // Ecriture sur le port série de la variable 'Vit' : la réponse
    }
    for (k = 0; k <= 24; k += 8) {
      Serial.write(sbpa_b[i] >> k);// Ecriture sur le port série de la variable 'sbpa_b[i]' : la commande
    }
    Shift_rad(); // shift (décalage) du registre à décalage
    rad[0] = sbpa_b[i]; // pour boucler le registre à décalage
    TempsCalcul = micros() - AncienTempsCalcul;
    delayMicroseconds(Te - TempsCalcul);
  }

  detachInterrupt(digitalPinToInterrupt(HALL_pin));
  analogWrite( MCC_pin  , 0); // arrêt du moteur

  Serial.end(); // fermeture de la connexion série
  digitalWrite(LED_R, LOW); // Fin du programme
  digitalWrite(LED_V, HIGH); // Led Rouge éteinte, Led Verte allumée
}

void loop () { // nécessaire pour tout programme Arduino

}

void TimeCount() {
  time = micros();
  Duree = time - AncienTemps;
  AncienTemps = time;
}

void Shift_rad() {
  rad[9] = rad[8];
  rad[8] = rad[7];
  rad[7] = rad[6];
  rad[6] = rad[5];
  rad[5] = rad[4];
  rad[4] = rad[3];
  rad[3] = rad[2];
  rad[2] = rad[1];
  rad[1] = rad[0];
}
